package edu.arbor.util.plugin;

import java.net.URL;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import net.sf.hibernate.HibernateException;
import net.sf.hibernate.Session;
import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.cfg.Configuration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionServlet;
import org.apache.struts.action.PlugIn;
import org.apache.struts.config.ModuleConfig;

/**
 * Implements the <code>PlugIn</code> interface to configure the Hibernate
 * data persistence library.  A configured
 * <code>net.sf.hibernate.SessionFactory</code> is stored in the 
 * <code>ServletContext</code> of the web application unless the property
 * <code>storedInServletContext</code> is set to <code>false</code>.
 * 
 * <p>
 * &lt;plugin class=&quot;net.sf.hibernate.plugins.struts.HibernatePlugIn&quot;&gt;
 *   &lt;set-property name=&quot;configFilePath&quot"
 *                value=&quot;path-to-config-file&quot;/&gt;
 *   &lt;set-property name=&quot;storedInServletContext&quot"
 *                value=&quot;true-or-false&quot;/&gt;
 * &lt;/plugin&gt;
 *
 * @author  <a href="mailto:bhandy@users.sf.net">Bradley M. Handy</a>
 * @version 1.0
 */
public class HibernatePlugIn implements PlugIn {

	public static final String SESSION_FACTORY_KEY =
		SessionFactory.class.getName();
	private static final ThreadLocal session = new ThreadLocal();

	private static Log _log = LogFactory.getLog(HibernatePlugIn.class);

	private boolean _storedInServletContext = true;
	private String _configFilePath = "/hibernate.cfg.xml";

	private ActionServlet _servlet = null;
	private ModuleConfig _config = null;
	private static SessionFactory _factory = null;

	public void destroy() {
		_servlet = null;
		_config = null;

		try {
			_log.debug("Destroying SessionFactory...");

			_factory.close();

			_log.debug("SessionFactory destroyed...");
		} catch (Exception e) {
			_log.error(
				"Unable to destroy SessionFactory...(exception ignored)",
				e);
		}
	}

	public void init(ActionServlet servlet, ModuleConfig config)
		throws ServletException {
		_servlet = servlet;
		_config = config;

		initHibernate();
	}

	private void initHibernate() throws ServletException {
		Configuration configuration = null;
		URL configFileURL = null;
		ServletContext context = null;

		try {
			configFileURL = HibernatePlugIn.class.getResource(_configFilePath);

			context = _servlet.getServletContext();

			if (_log.isDebugEnabled()) {
				_log.debug(
					"Initializing Hibernate from " + _configFilePath + "...");
			}

			configuration = (new Configuration()).configure(configFileURL);
			_factory = configuration.buildSessionFactory();

/*
			if (_storedInServletContext) {
				_log.debug("Storing SessionFactory in ServletContext...");

				context.setAttribute(SESSION_FACTORY_KEY, _factory);
			}
*/			
		} catch (Throwable t) {
			_log.error("Exception while initializing Hibernate.");
			_log.error("Rethrowing exception...", t);

			throw (new ServletException(t));
		}
	}

	public void setConfigFilePath(String configFilePath) {
		if ((configFilePath == null)
			|| (configFilePath.trim().length() == 0)) {
			throw new IllegalArgumentException("configFilePath cannot be blank or null.");
		}

		if (_log.isDebugEnabled()) {
			_log.debug(
				"Setting 'configFilePath' to '" + configFilePath + "'...");
		}

		_configFilePath = configFilePath;
	}

	public void setStoredInServletContext(String storedInServletContext) {
		if ((storedInServletContext == null)
			|| (storedInServletContext.trim().length() == 0)) {
			storedInServletContext = "false";
		}

/*
		if (_log.isDebugEnabled()) {
			_log.debug(
				"Setting 'storedInServletContext' to '"
					+ storedInServletContext
					+ "'...");
		}
*/
		_storedInServletContext =
			new Boolean(storedInServletContext).booleanValue();
	}

	public static void closeSession() throws HibernateException {
		Session s = (Session) session.get();
		session.set(null);
		if (s != null)
			s.close();
	}

	public static Session openSession() throws HibernateException {
		Session s = (Session) session.get();
		if (s == null) {
			s = _factory.openSession();
			session.set(s);
		}
		return (s);
	}
}